part of masamune_purchase_stripe;

@immutable
class StripeMarketPlaceAdapter extends MarketPlaceAdapter<
    StripeConnectPurchaseProduct,
    StripeConnectPurchaseOptions,
    FirestoreDynamicDocumentModel,
    FirestoreDynamicCollectionModel,
    FirestoreDynamicCollectionModel> {
  const StripeMarketPlaceAdapter({
    required this.revenue,
    required this.serverPath,
    required this.callbackUrl,
    required this.supportEmail,
    this.view = const StripeDefaultView(),
    this.viewWithoutBack = const StripeDefaultViewWithoutBack(),
    this.currency = StripeCurrency.usd,
    this.debugUserId,
  });

  final String serverPath;
  final String callbackUrl;
  final String? debugUserId;
  final double revenue;
  final StripeCurrency currency;
  final String supportEmail;
  final ValueWidget<Widget> view;
  final ValueWidget<Widget> viewWithoutBack;

  static late final BuildContext _context;

  @override
  Future<void> initialize() async {
    await StripeConnectCore.initialize(
      serverPath: serverPath,
      callbackUrl: callbackUrl,
      revenue: revenue,
      supportEmail: supportEmail,
      debugUserId: debugUserId,
      currency: currency,
    );
  }

  @override
  Future<void> cancel(String orderId) async {
    final stripePurchase = read(stripeConnectPurchaseModelProvider);
    await stripePurchase.cancel(_context, orderId: orderId);
  }

  @override
  Future<void> capture(
    BuildContext context,
    String orderId, {
    StripeConnectPurchaseOptions? options,
  }) async {
    final stripePurchase = read(stripeConnectPurchaseModelProvider);
    await stripePurchase.capture(
      _context,
      orderId: orderId,
      online: options?.online ?? true,
      amount: options?.amount,
    );
  }

  @override
  Future<void> confirm(
    String orderId, {
    StripeConnectPurchaseOptions? options,
  }) async {
    final stripePurchase = read(stripeConnectPurchaseModelProvider);
    await stripePurchase.confirm(
      _context,
      orderId: orderId,
      online: options?.online ?? true,
      popWhenClosedByWebview: true,
      builder: (webView, onClosed) async {
        await UIModal.show(
          _context,
          disableBackKey: true,
          child: ValueProvider<Widget>(
            value: webView,
            child: viewWithoutBack,
          ),
        );
      },
    );
  }

  @override
  Future<void> deletePaymentMethod(
    String paymentMethodId,
  ) async {
    final stripePayment = read(stripeConnectPaymentModelProvider);
    await stripePayment.delete(paymentMethodId);
  }

  @override
  Future<void> purchase(
    StripeConnectPurchaseProduct product,
  ) async {
    final stripePurchase = read(stripeConnectPurchaseModelProvider);
    await stripePurchase.create(
      _context,
      orderId: product.orderId,
      amount: product.amount,
      targetUserId: product.targetUserId,
      description: product.description,
      locale: product.locale,
    );
  }

  @override
  Future<void> refresh(String orderId) async {
    final stripePurchase = read(stripeConnectPurchaseModelProvider);
    await stripePurchase.refresh(_context, orderId: orderId);
  }

  @override
  Future<MarketPlaceStatus> registrationSeller() async {
    final stripeAccount = read(stripeConnectAccountModelProvider);
    return await stripeAccount.create(_context, popWhenClosedByWebview: true,
        builder: (webView, onSuccess, onCancel) async {
      await UIModal.show(
        _context,
        child: ValueProvider(value: webView, child: view),
      );
    });
  }

  @override
  Future<void> setDefaultPaymentMethod(
    String paymentMethodId,
  ) async {
    final stripePayment = read(stripeConnectPaymentModelProvider);
    await stripePayment.setDefault(paymentMethodId);
  }

  @override
  Future<MarketPlaceStatus> setupPaymentMethod() async {
    final stripePayment = read(stripeConnectPaymentModelProvider);
    return await stripePayment.create(_context, popWhenClosedByWebview: true,
        builder: (webView, onSuccess, onCancel) async {
      await UIModal.show(
        _context,
        child: ValueProvider(value: webView, child: view),
      );
    });
  }

  @override
  Future<void> showDashboard() async {
    final stripeAccount = read(stripeConnectAccountModelProvider);
    await stripeAccount.dashboard(_context, popWhenClosedByWebview: true,
        builder: (webView, onClosed) async {
      await UIModal.show(
        _context,
        child: ValueProvider(value: webView, child: view),
      );
    });
  }

  @override
  FirestoreDynamicDocumentModel? watchAccountDocument(WidgetRef ref) {
    return ref.watch(stripeConnectAccountModelProvider).value;
  }

  @override
  FirestoreDynamicCollectionModel? watchPaymentMethodCollection(WidgetRef ref) {
    return ref.watch(stripeConnectPaymentModelProvider).value;
  }

  @override
  FirestoreDynamicCollectionModel? watchPurchaseCollection(WidgetRef ref) {
    return ref.watch(stripeConnectPurchaseModelProvider).value;
  }

  @override
  Future<void> onInit(BuildContext context) {
    _context = context.navigator.context;
    return super.onInit(context);
  }
}

class StripeDefaultView extends ValueWidget<Widget> {
  const StripeDefaultView();
  @override
  Widget build(BuildContext context, WidgetRef ref, Widget value) {
    return Scaffold(
      appBar: AppBar(),
      body: value,
    );
  }
}

class StripeDefaultViewWithoutBack extends ValueWidget<Widget> {
  const StripeDefaultViewWithoutBack();
  @override
  Widget build(BuildContext context, WidgetRef ref, Widget value) {
    return Scaffold(
      body: value,
    );
  }
}
